using namespace System;
using namespace System::Windows::Forms;
using namespace System::Drawing;
using namespace System::Drawing::Drawing2D;
using namespace System::Collections::Generic;
using namespace System::Runtime::InteropServices;
using namespace System::IO;

#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>
#include <iterator>

using namespace std;

#include "Graph.h"

namespace cs493
{
	Graph::Graph(int width, int height)
	{
		Width = width;
		Height = height;
		floatArray_ = gcnew List<PointF>;
	}

	void Graph::readDataFile(String^ filename)
	{
		
		StreamReader^ infile = File::OpenText(filename);
		String^ nextLine;

		const float MAX_X = 100.0f;
		float x = 0;
		float y = 0;
		while (x >= 0)
		{
			nextLine = infile->ReadLine();
			x = Convert::ToSingle(nextLine);
			nextLine = infile->ReadLine();
			y = Convert::ToSingle(nextLine);
			floatArray_->Add(PointF(x,y));
		}
	}

	void Graph::OnResize(EventArgs^ args)
   {
	   this->Invalidate();
   }

   void Graph::OnPaint(PaintEventArgs^ args) 
   {
      // Get the instance of the Graphics class for this window.
      Graphics^ graphics = args->Graphics;

	  SizeF visibleBounds = graphics->VisibleClipBounds.Size;
	  int windowWidth = static_cast<int>(visibleBounds.Width);
	  int windowHeight = static_cast<int>(visibleBounds.Height);
	  int helloX = windowWidth/2;
	  int helloY = windowHeight/2;
	  const float PIXELS_PER_POINT = 4.0f;

	  /*
	  int nextEndPoint = 1;
	  while(nextEndPoint < floatArray_->Count)
	  {
	
		  graphics->FillRectangle(Brushes::Blue, RectangleF(floatArray_[nextEndPoint].X,
			  floatArray_[nextEndPoint].Y, PIXELS_PER_POINT, PIXELS_PER_POINT));

		  	  graphics->DrawLine(Pens::Red, floatArray_[nextEndPoint-1],
				floatArray_[nextEndPoint]);	
			  ++nextEndPoint;
	  }
	  */
	
	  //graphics->PageUnit = GraphicsUnit::Inch;

	  //graphics->FillRectangle(Brushes::BlanchedAlmond, 0,0,
		//  windowWidth-1, windowHeight-1);

	  //const float PIXELS_PER_INCH = 1000;
	  //Pen^ thinPen = gcnew Pen(Color::Blue, 1.0/PIXELS_PER_INCH);
	  //graphics->DrawLine(thinPen,0,0,1,1);

	  /*
	  const int X_PIXEL_WIDTH = 10;
	  Pen^ xPen = gcnew Pen(Color::DarkSalmon, X_PIXEL_WIDTH);
	  graphics->DrawLine(xPen,0,0,windowWidth-1, windowHeight-1);
	  graphics->DrawLine(xPen,windowWidth-1, 0, 0, windowHeight-1);

	  graphics->DrawEllipse(Pens::DodgerBlue, RectangleF(0,0,windowWidth-1, 
		                                                 windowHeight-1));
      graphics->DrawString("Hello World!", 
		  gcnew Drawing::Font("Arial", 16), Brushes::DarkOliveGreen, 
		  helloX, helloY);
		  */

	  // World Coordinate system is the system that we use to measure the real-world
	  // instances of the things we are drawing.  It is best if the application
	  // can be written using this coordinate system.

	  // Page Coordinate system describes physical units of measure on the device 
	  // itself.  

	  // Device Coordinate system is the system that device itself uses.
	
	  const float X_MARGIN_PERCENT = .15f;
	  const float Y_MARGIN_PERCENT = .2f;

	  const float xWorld = 4.0f * Math::PI; 
	  const float yWorld = 3.0f;

	  float yClip = windowHeight - (windowHeight * Y_MARGIN_PERCENT);
	  float xClip = windowWidth - (windowWidth * X_MARGIN_PERCENT);
	  float yMargin = windowHeight * Y_MARGIN_PERCENT / 2.0f;
	  float xMargin = windowWidth * X_MARGIN_PERCENT / 2.0f ;

	  RectangleF^ clipRect = gcnew RectangleF(0.0f, 0.0f, xClip, yClip);

	  // Get the size of the client area in pixels
	  SizeF^ sizef = graphics->VisibleClipBounds.Size;

	  // Set the clip rectangle
	  graphics->SetClip(*clipRect);

	  // Center the clip rectangle
	  graphics->TranslateClip(xMargin, yMargin);

	  // We need to translate everything we draw by this amount as well
	  graphics->TranslateTransform(xMargin, yMargin);

	  // Specify the scale factor.  This is the number of pixels per unit in world coordinates
	  graphics->ScaleTransform(xClip/xWorld, yClip/yWorld);

	  // Place the center of the world coordinate system at zero.
	  // 0,0) was initially at the top left, so we have to move it
      // to the right by half the maximum world x-coordinate, and move
      // it down by half the maximum world y-coordinate.
      graphics->TranslateTransform(xWorld/2, yWorld/2);

      // Flip the y-axis so that y increases as you go up the display.
      // Another way to do this would be to change the previous two
      // calls to:
      // graphics.ScaleTransform(xClip/xWorld, 
      //                            -yClip/yWorld);
      // graphics.TranslateTransform(xWorld/2, -yWorld/2);
      graphics->ScaleTransform(1,-1);

      // Create pens. One world coordinate would be much to big
      // for the pen width, so we divide the max world x-coordinate by
      // the width (in pixels) of the viewport.  This gives us
      // the approximate size of a single pixel in world coordinates.
	  Pen^ dataPen = gcnew Pen(Color::Red, xWorld/xClip);
	  Pen^ axisPen = gcnew Pen(Color::Gray, xWorld/xClip);
	  Pen^ cosPen = gcnew Pen(Color::Blue, xWorld/xClip);

	   // Draw the x axis
      graphics->DrawLine(axisPen, -xWorld/2.0f, 0.0f, xWorld/2.0f, 0.0f);

	  int nPoints = 1000;

      // Draw the y axis
      graphics->DrawLine(axisPen, 0.0f, -yWorld/2.0f, 0.0f, yWorld/2.0f);
         
	    // Change in theta per point
      float deltaTheta = xWorld / nPoints;

      // Start value for theta
	  float theta = -2.0f * Math::PI;

      // Coordinate of previous point plotted.
      float lastX = theta;
      float lastY = 0;

      // Frequency of sine wave.
      float frequency = 3;
            
	  while(theta <= 2*Math::PI)
      {
        // Draw a line from previous point to new point
        graphics->DrawLine(dataPen, lastX, lastY, theta, 
			(float) Math::Sin(frequency*theta));

        // Make this point the previous point
        lastX = theta;
		lastY = (float) Math::Sin(frequency*theta);

        theta = theta + deltaTheta;
      }

      // Draw tic marks
      const float TIC_PERCENTAGE = .03f;
         
      int firstXTic = (int) -(xWorld/2);
      int lastXTic = (int) xWorld;
      float yTicLength = yWorld * TIC_PERCENTAGE;

      // Draw the tic marks on the x-axis
      int i;
      for(i=firstXTic;i<=lastXTic;i++)
      {
        graphics->DrawLine(axisPen, static_cast<float>(i), 
			static_cast<float>(-yTicLength/2), static_cast<float>(i), 
			static_cast<float>(yTicLength/2));
      }

      // Draw the tic marks on the y-axis
      int firstYTic = (int) -(yWorld/2);
      int lastYTic = (int) yWorld;
      float xTicLength = xWorld * TIC_PERCENTAGE;
      for(i=firstYTic;i<=lastYTic;i++)
      {
        graphics->DrawLine(axisPen, 
			static_cast<float>(-xTicLength/2), 
			static_cast<float>(i), 
			static_cast<float>(xTicLength/2), static_cast<float>(i));
      }

	        // Save this graphics state so that we can return to it later.
      GraphicsState^ savedGraphicsState = graphics->Save();

      // Reset to the initial state (all our transformations are
      // undone!)
      graphics->ResetTransform();
      graphics->ResetClip();
         
      // Draw a box around the whole thing.
	  graphics->DrawRectangle(Pens::Black,
                             xMargin,yMargin,
                             xClip,yClip);

	  StringFormat^ titleFormat = gcnew StringFormat();
	  titleFormat->Alignment = StringAlignment::Center;
	  //titleFormat->Alignment(StringAlignment::Center);
	 

	  const float TITLE_FONT_HEIGHT_OVER_WINDOW_HEIGHT = .05;
	  int titleFontSize = yClip * TITLE_FONT_HEIGHT_OVER_WINDOW_HEIGHT;
	  Drawing::Font^ titleFont = gcnew Drawing::Font("Arial", titleFontSize);
	 
	  graphics->DrawString("Cos and Sin", titleFont, Brushes::Black, windowWidth/2, 
		  yMargin+yClip, titleFormat);

      // Now we add the cosine to the graph.
      // First restore the transforms to the saved state.
      graphics->Restore(savedGraphicsState);


      // Draw the cos
	  theta = -2.0f * Math::PI;
      lastX = theta;
	  lastY = static_cast<float>(Math::Cos(frequency*theta));

	  while(theta <= 2*Math::PI)
      {
        // Draw a line from previous point to new point
        graphics->DrawLine(cosPen, lastX, lastY, theta, 
			static_cast<float>(Math::Cos(frequency*theta)));

        // Make this point the previous point
        lastX = theta;
		lastY = static_cast<float>( Math::Cos(frequency*theta));

        theta = theta + deltaTheta;
      }


	  int nextEndPoint = 1;
	  while(nextEndPoint < floatArray_->Count -1)
	  {
	
		  graphics->FillRectangle(Brushes::Blue, RectangleF(floatArray_[nextEndPoint].X,
			  floatArray_[nextEndPoint].Y, xWorld/xClip*4, yWorld/yClip*4));

		  	  graphics->DrawLine(dataPen, floatArray_[nextEndPoint-1],
				floatArray_[nextEndPoint]);	
			  ++nextEndPoint;
	  }


   }
} 

